@@ -4,16 +4,16 @@
  * Input of Calma GDS-II stream format.
  * Processing for cells.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -47,6 +47,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "calma/calma.h"
 
 int calmaNonManhattan;
+int CalmaFlattenLimit = 10;
 
 extern HashTable calmaDefInitHash;
 
@@ -255,7 +256,7 @@ calmaExact()
 }
 
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -349,6 +350,7 @@ calmaParseStructure(filename)
     DBCellSetAvail(cifReadCellDef);
     HashSetValue(he, cifReadCellDef);
     cifCurReadPlanes = cifSubcellPlanes;
+    cifReadCellDef->cd_flags &= ~CDDEREFERENCE;
 
     /* Done with strname */
     if (strname != NULL) freeMagic(strname);
@@ -409,7 +411,8 @@ calmaParseStructure(filename)
      * cell by painting when instanced.  But---if this cell was
      * instanced before it was defined, then it can't be flattened.
      */
-    if (CalmaFlattenUses && (!was_called) && (npaths < 10) && (nsrefs == 0))
+    if (CalmaFlattenUses && (!was_called) && (npaths < CalmaFlattenLimit)
+		&& (nsrefs == 0))
     {
 	/* If CDFLATGDS is already set, may need to remove	*/
 	/* existing planes and free memory.			*/
@@ -480,7 +483,7 @@ syntaxerror:
     return (calmaSkipTo(CALMA_ENDSTR));
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -527,7 +530,7 @@ calmaParseElement(filename, pnsrefs, pnpaths)
 	    if (madeinst >= 0)
 		(*pnsrefs) += madeinst;
 	    break;
-	case CALMA_BOUNDARY:	
+	case CALMA_BOUNDARY:
 	    calmaElementBoundary();
 	    (*pnpaths)++;
 	    break;
@@ -535,11 +538,11 @@ calmaParseElement(filename, pnsrefs, pnpaths)
 	    calmaElementBox();
 	    (*pnpaths)++;
 	    break;
-	case CALMA_PATH:	
+	case CALMA_PATH:
 	    calmaElementPath();
 	    (*pnpaths)++;
 	    break;
-	case CALMA_TEXT:	
+	case CALMA_TEXT:
 	    calmaElementText();
 	    break;
 	case CALMA_NODE:
@@ -664,6 +667,10 @@ calmaElementSref(filename)
 	    TxPrintf("Cell definition %s does not exist!\n", sname);
 	    fseek(calmaInputFile, originalFilePos, SEEK_SET);
 	    def = calmaFindCell(sname, NULL);
+	    /* Cell flags set to "dereferenced" in case there is no	*/
+	    /* definition in the GDS file.  If there is a definition	*/
+	    /* made after the instance, then the flag will be cleared.	*/
+	    def->cd_flags |= CDDEREFERENCE;
 	}
     }
 
@@ -962,7 +969,7 @@ calmaElementSref(filename)
 
     return ((def->cd_flags & CDFLATGDS) ? (madeinst ? 1 : 0) : 1);
 }
-
+
 
 /* Callback function for determining if a cell has at least one subcell */
 
@@ -1001,7 +1008,7 @@ gdsCopyPaintFunc(tile, gdsCopyRec)
 
     DBNMPaintPlane(plane, dinfo, &targetRect, CIFPaintTable,
 		(PaintUndoInfo *)NULL);
-    
+
     return 0;
 }
 
@@ -1043,7 +1050,7 @@ calmaFindCell(name, was_called)
 	def = DBCellLookDef(name);
 	if (def == NULL)
 	{
-	    def = DBCellNewDef(name, (char *) NULL);
+	    def = DBCellNewDef(name);
 
 	    /*
 	     * Tricky point:  call DBReComputeBbox here to make SURE
