@@ -4,16 +4,16 @@
  * Input of Calma GDS-II stream format.
  * Processing of paint (paths, boxes, and boundaries) and text.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -57,8 +57,6 @@ extern HashTable calmaDefInitHash;
 extern void calmaLayerError();
 bool calmaReadPath();
 
-typedef enum { LABEL_TYPE_NONE, LABEL_TYPE_TEXT, LABEL_TYPE_PORT } labelType;
-
 /*
  * ----------------------------------------------------------------------------
  *
@@ -100,7 +98,9 @@ calmaInputRescale(n, d)
 	{
 	    /* Scale the GDS planes in this cell's cd_client record */
 	    Plane **gdsplanes = (Plane **)def->cd_client;
-	    CIFScalePlanes(n, d, gdsplanes);
+	    /* Should not happen, but punt if client record is not set */
+	    if (def->cd_client != (ClientData)CLIENTDEFAULT)
+		CIFScalePlanes(n, d, gdsplanes);
 	}
     }
 
@@ -145,7 +145,7 @@ calmaReadPoint(p, iscale)
 	rescale = calmaReadScale2 / FindGCF(calmaReadScale2, abs(p->p_x));
 	if ((calmaReadScale1 * rescale) > CIFRescaleLimit)
 	{
-	    calmaReadError("Warning:  calma units at max scale; value rounded\n");
+	    CalmaReadError("Warning:  calma units at max scale; value rounded\n");
 	    if (p->p_x < 0)
 		p->p_x -= ((calmaReadScale2 - 1) >> 1);
 	    else
@@ -167,7 +167,7 @@ calmaReadPoint(p, iscale)
 	rescale = calmaReadScale2 / FindGCF(calmaReadScale2, abs(p->p_y));
 	if ((calmaReadScale1 * rescale) > CIFRescaleLimit)
 	{
-	    calmaReadError("Warning:  calma units at max scale; value rounded\n");
+	    CalmaReadError("Warning:  calma units at max scale; value rounded\n");
 	    if (p->p_y < 0)
 		p->p_y -= ((calmaReadScale2 - 1) >> 1);
 	    else
@@ -184,7 +184,7 @@ calmaReadPoint(p, iscale)
     p->p_y /= calmaReadScale2;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -218,7 +218,7 @@ calmaElementBoundary()
     if (!calmaReadI2Record(CALMA_LAYER, &layer)
 	    || !calmaReadI2Record(CALMA_DATATYPE, &dt))
     {
-	calmaReadError("Missing layer or datatype in boundary/box.\n");
+	CalmaReadError("Missing layer or datatype in boundary/box.\n");
 	return;
     }
 
@@ -233,10 +233,10 @@ calmaElementBoundary()
 	plane = cifCurReadPlanes[ciftype];
 
     /* Read the path itself, building up a path structure */
-    if (!calmaReadPath(&pathheadp, (plane == NULL) ? 0 : 1)) 
+    if (!calmaReadPath(&pathheadp, (plane == NULL) ? 0 : 1))
     {
 	if (plane != NULL)
-	    calmaReadError("Error while reading path for boundary/box; ignored.\n");
+	    CalmaReadError("Error while reading path for boundary/box; ignored.\n");
 	return;
     }
 
@@ -274,7 +274,6 @@ calmaElementBoundary()
 	}
     }
 
-    CIFPropRecordPath(cifReadCellDef, pathheadp, FALSE);
     rp = CIFPolyToRects(pathheadp, plane, CIFPaintTable, (PaintUndoInfo *)NULL);
     CIFFreePath(pathheadp);
 
@@ -289,8 +288,18 @@ calmaElementBoundary()
 
     if (rp != NULL)
     {
+        Rect rpc;
+	int savescale;
+
+	/* Convert rp to magic database units to compare to label rects */
+        rpc = rp->r_r;
+	rpc.r_xbot /= cifCurReadStyle->crs_scaleFactor;
+	rpc.r_xtop /= cifCurReadStyle->crs_scaleFactor;
+	rpc.r_ybot /= cifCurReadStyle->crs_scaleFactor;
+	rpc.r_ytop /= cifCurReadStyle->crs_scaleFactor;
+
 	if ((ciftype >= 0) &&
-		((cifCurReadStyle->crs_labelSticky[ciftype] != LABEL_TYPE_NONE)))
+		(cifCurReadStyle->crs_labelSticky[ciftype] != LABEL_TYPE_NONE))
 	{
 	    Label *lab;
 	    TileType type;
@@ -298,12 +307,17 @@ calmaElementBoundary()
 	    type = cifCurReadStyle->crs_labelLayer[ciftype];
 	    for (lab = cifReadCellDef->cd_labels; lab; lab = lab->lab_next)
 	    {
-		if ((GEO_SURROUND(&rp->r_r, &lab->lab_rect)) && (lab->lab_type == type))
+		if ((GEO_SURROUND(&rpc, &lab->lab_rect)) && (lab->lab_type == type))
 		{
-		    lab->lab_rect = rp->r_r;	/* Replace with larger rectangle */
+		    lab->lab_rect = rpc;	/* Replace with larger rectangle */
 		    break;
 		}
 	    }
+	    if (lab == NULL)
+	    {
+		/* There was no label in the area.  Create a placeholder label */
+		DBPutLabel(cifReadCellDef, &rpc, GEO_CENTER, "", type, 0);
+	    }
 	}
     }
 
@@ -317,7 +331,7 @@ calmaElementBoundary()
 
     if (cifCurReadPlanes == cifEditCellPlanes)
     {
-	CIFPaintCurrent();
+	CIFPaintCurrent(FILE_CALMA);
 	DBReComputeBbox(cifReadCellDef);
 	DRCCheckThis(cifReadCellDef, TT_CHECKPAINT, &cifReadCellDef->cd_bbox);
 	DBWAreaChanged(cifReadCellDef, &cifReadCellDef->cd_bbox,
@@ -333,7 +347,7 @@ calmaElementBoundary()
 	DBPlaceCell(use, cifReadCellDef);
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -371,7 +385,7 @@ calmaElementBox()
     if (!calmaReadI2Record(CALMA_LAYER, &layer)
 	    || !calmaReadI2Record(CALMA_BOXTYPE, &dt))
     {
-	calmaReadError("Missing layer or datatype in boundary/box.\n");
+	CalmaReadError("Missing layer or datatype in boundary/box.\n");
 	return;
     }
 
@@ -395,7 +409,7 @@ calmaElementBox()
     READRH(nbytes, rtype);
     if (nbytes < 0)
     {
-	calmaReadError("EOF when reading box.\n");
+	CalmaReadError("EOF when reading box.\n");
 	return;
     }
     if (rtype != CALMA_XY)
@@ -408,7 +422,7 @@ calmaElementBox()
     npoints = (nbytes - CALMAHEADERLENGTH) / 8;
     if (npoints != 5)
     {
-	calmaReadError("Box doesn't have 5 points.\n");
+	CalmaReadError("Box doesn't have 5 points.\n");
 	(void) calmaSkipBytes(nbytes - CALMAHEADERLENGTH);
 	return;
     }
@@ -479,7 +493,7 @@ calmaElementPath()
     if (pathtype != CALMAPATH_SQUAREFLUSH && pathtype != CALMAPATH_SQUAREPLUS &&
 		pathtype != CALMAPATH_CUSTOM)
     {
-	calmaReadError("Warning: pathtype %d unsupported (ignored).\n", pathtype);
+	CalmaReadError("Warning: pathtype %d unsupported (ignored).\n", pathtype);
 	pathtype = CALMAPATH_SQUAREFLUSH;
     }
 
@@ -488,18 +502,18 @@ calmaElementPath()
      * Allow zero-width paths; we will ignore them later.
      */
     width = 0;
-    PEEKRH(nbytes, rtype) 
+    PEEKRH(nbytes, rtype)
     if (nbytes > 0 && rtype == CALMA_WIDTH)
     {
-	if (!calmaReadI4Record(CALMA_WIDTH, &width)) 
+	if (!calmaReadI4Record(CALMA_WIDTH, &width))
 	{
-	    calmaReadError("Error in reading WIDTH in calmaElementPath()\n") ;
+	    CalmaReadError("Error in reading WIDTH in calmaElementPath()\n") ;
 	    return;
 	}
     }
     width *= calmaReadScale1;
     if (width % calmaReadScale2 != 0)
-	calmaReadError("Wire width snapped to nearest integer boundary.\n");
+	CalmaReadError("Wire width snapped to nearest integer boundary.\n");
 
     width /= calmaReadScale2;
 
@@ -516,12 +530,12 @@ calmaElementPath()
     if (nbytes > 0 && rtype == CALMA_BGNEXTN)
     {
 	if (!calmaReadI4Record(CALMA_BGNEXTN, &extend1))
-	    calmaReadError("Error in reading BGNEXTN in path (ignored)\n") ;
+	    CalmaReadError("Error in reading BGNEXTN in path (ignored)\n") ;
 	else
 	{
 	    extend1 *= calmaReadScale1;
 	    if (extend1 % calmaReadScale2 != 0)
-		calmaReadError("Wire extension snapped to nearest integer boundary.\n");
+		CalmaReadError("Wire extension snapped to nearest integer boundary.\n");
 	    extend1 *= 2;
 	    extend1 /= calmaReadScale2;
 	}
@@ -531,12 +545,12 @@ calmaElementPath()
     if (nbytes > 0 && rtype == CALMA_ENDEXTN)
     {
 	if (!calmaReadI4Record(CALMA_ENDEXTN, &extend2))
-	    calmaReadError("Error in reading ENDEXTN in path (ignored)\n") ;
+	    CalmaReadError("Error in reading ENDEXTN in path (ignored)\n") ;
 	else
 	{
 	    extend2 *= calmaReadScale1;
 	    if (extend2 % calmaReadScale2 != 0)
-		calmaReadError("Wire extension snapped to nearest integer boundary.\n");
+		CalmaReadError("Wire extension snapped to nearest integer boundary.\n");
 	    extend2 *= 2;
 	    extend2 /= calmaReadScale2;
 	}
@@ -544,9 +558,9 @@ calmaElementPath()
 
     /* Read the points in the path */
     savescale = calmaReadScale1;
-    if (!calmaReadPath(&pathheadp, 2)) 
+    if (!calmaReadPath(&pathheadp, 2))
     {
-	calmaReadError("Improper path; ignored.\n");
+	CalmaReadError("Improper path; ignored.\n");
 	return;
     }
     if (savescale != calmaReadScale1)
@@ -637,14 +651,14 @@ calmaElementPath()
 	    }
 	}
 
-	CIFPropRecordPath(cifReadCellDef, pathheadp, TRUE);
+	CIFPropRecordPath(cifReadCellDef, pathheadp, TRUE, "path");
 	CIFPaintWirePath(pathheadp, width,
 		(pathtype == CALMAPATH_SQUAREFLUSH || pathtype == CALMAPATH_CUSTOM) ?
 		FALSE : TRUE, plane, CIFPaintTable, (PaintUndoInfo *)NULL);
 
 	if (cifCurReadPlanes == cifEditCellPlanes)
 	{
-	    CIFPaintCurrent();
+	    CIFPaintCurrent(FILE_CALMA);
 	    DBReComputeBbox(cifReadCellDef);
 	    DRCCheckThis(cifReadCellDef, TT_CHECKPAINT, &cifReadCellDef->cd_bbox);
 	    DBWAreaChanged(cifReadCellDef, &cifReadCellDef->cd_bbox,
@@ -700,7 +714,7 @@ calmaElementText()
     cifnum = CIFCalmaLayerToCifLayer(layer, textt, cifCurReadStyle);
     if (cifnum < 0)
     {
-	if(cifCurReadStyle->crs_flags & CRF_IGNORE_UNKNOWNLAYER_LABELS) 
+	if (cifCurReadStyle->crs_flags & CRF_IGNORE_UNKNOWNLAYER_LABELS)
 	    type = -1;
 	else {
 	    calmaLayerError("Label on unknown layer/datatype", layer, textt);
@@ -713,7 +727,7 @@ calmaElementText()
     angle = 0;
 
     /* Default size is 1um */
-    size = (int)((800 * cifCurReadStyle->crs_multiplier)
+    size = (int)((1000 * cifCurReadStyle->crs_multiplier)
 				/ cifCurReadStyle->crs_scaleFactor);
     /* Default position is bottom-right (but what the spec calls "top-left"!) */
     pos = GEO_SOUTHEAST;
@@ -760,6 +774,31 @@ calmaElementText()
     else if (nbytes > 0 && rtype != CALMA_STRANS)
 	calmaSkipSet(ignore);
 
+    /* NOTE:  Record may contain both PRESENTATION and WIDTH */
+    PEEKRH(nbytes, rtype);
+    if (nbytes > 0 && rtype == CALMA_WIDTH)
+    {
+	int width;
+
+	/* Use WIDTH value to set the font size */
+	if (!calmaReadI4Record(CALMA_WIDTH, &width))
+	{
+	    CalmaReadError("Error in reading WIDTH in calmaElementText()\n") ;
+	    return;
+	}
+	width *= calmaReadScale1;
+	if (width % calmaReadScale2 != 0)
+	    CalmaReadError("Text width snapped to nearest integer boundary.\n");
+
+        width /= calmaReadScale2;
+
+	/* Convert to database units, because dimension goes to PutLabel    */
+	/* and is not converted through CIFPaintCurrent().		    */
+	size = CIFScaleCoord(width, COORD_ANY);
+    }
+    else if (nbytes > 0 && rtype != CALMA_STRANS)
+	calmaSkipSet(ignore);
+
     READRH(nbytes, rtype);
     if (nbytes > 0 && rtype == CALMA_STRANS)
     {
@@ -770,9 +809,10 @@ calmaElementText()
 	if (nbytes > 0 && rtype == CALMA_MAG)
 	{
 	    calmaReadR8(&dval);
+
 	    /* Assume that MAG is the label size in microns	*/
-	    /* "size" is the label size in 8 * (database units) */
-	    size = (int)((dval * 800 * cifCurReadStyle->crs_multiplier)
+	    /* "size" is the label size in 10 * (database units) */
+	    size = (int)((dval * 1000 * cifCurReadStyle->crs_multiplier)
 				/ cifCurReadStyle->crs_scaleFactor);
 	}
 	else
@@ -802,7 +842,7 @@ calmaElementText()
     nbytes -= CALMAHEADERLENGTH;
     if (nbytes < 8)
     {
-	calmaReadError("Not enough bytes in point record.\n");
+	CalmaReadError("Not enough bytes in point record.\n");
     }
     else
     {
@@ -825,7 +865,7 @@ calmaElementText()
      * a flag for that in the "cifoutput" section of the techfile.
      */
 
-#if 0 
+#if 0
 
     {
 	static bool algmsg = FALSE;
@@ -834,7 +874,7 @@ calmaElementText()
 	char *savstring;
 	for (cp = textbody; *cp; cp++)
 	{
-	    if (*cp <= ' ' | *cp > '~') 
+	    if (*cp <= ' ' | *cp > '~')
 	    {
 		if (!changed)
 		{
@@ -843,7 +883,7 @@ calmaElementText()
 		}
 		if (*cp == '\r' && *(cp+1) == '\0')
 		    *cp = '\0';
-		else if (*cp == '\r') 
+		else if (*cp == '\r')
 		    *cp = '_';
 		else if (*cp == ' ')
 		    *cp = '_';
@@ -852,15 +892,15 @@ calmaElementText()
 	    }
 	}
 	if (changed) {
-	    calmaReadError("Warning:  improper characters fixed in label '%s'\n",
+	    CalmaReadError("Warning:  improper characters fixed in label '%s'\n",
 			savstring);
 	    if (!algmsg) {
 		algmsg = TRUE;
-		calmaReadError("    (algorithm used:  trailing <CR> dropped, "
+		CalmaReadError("    (algorithm used:  trailing <CR> dropped, "
 				"<CR> and ' ' changed to '_', \n"
 				"    other non-printables changed to '?')\n");
 	    }
-	    calmaReadError("	modified label is '%s'\n", textbody);
+	    CalmaReadError("	modified label is '%s'\n", textbody);
 	    freeMagic(savstring);
 	}
     }
@@ -870,21 +910,29 @@ calmaElementText()
     /* Place the label */
     if (strlen(textbody) == 0)
     {
-	calmaReadError("Warning:  Ignoring empty string label at (%d, %d)\n",
+	CalmaReadError("Warning:  Ignoring empty string label at (%d, %d)\n",
 		r.r_ll.p_x * cifCurReadStyle->crs_scaleFactor,
 		r.r_ll.p_y * cifCurReadStyle->crs_scaleFactor);
     }
+    else if (cifCurReadStyle->crs_labelSticky[cifnum] == LABEL_TYPE_CELLID)
+    {
+	/* Special handling of label layers marked "cellid" in the techfile. */
+	/* The actual cellname is the ID string, not the GDS structure name. */
+	DBCellRenameDef(cifReadCellDef, textbody);
+    }
     else if (type < 0)
     {
-	calmaReadError("Warning:  label \"%s\" at (%d, %d) is on unhandled"
-		" layer:purpose pair %d:%d and will be discarded.\n", textbody,
-		r.r_ll.p_x * cifCurReadStyle->crs_scaleFactor,
-		r.r_ll.p_y * cifCurReadStyle->crs_scaleFactor, layer, textt);
+	if (!(cifCurReadStyle->crs_flags & CRF_IGNORE_UNKNOWNLAYER_LABELS))
+	    CalmaReadError("Warning:  label \"%s\" at (%d, %d) is on unhandled"
+		    " layer:purpose pair %d:%d and will be discarded.\n", textbody,
+		    r.r_ll.p_x * cifCurReadStyle->crs_scaleFactor,
+		    r.r_ll.p_y * cifCurReadStyle->crs_scaleFactor, layer, textt);
     }
     else
     {
 	int flags, i;
 	Label *lab;
+	Label *sl;
 
         if (type == TT_SPACE)
 	    /* Assigning GDS layer to space prevents making the label sticky */
@@ -896,6 +944,31 @@ calmaElementText()
 	else
 	    flags = 0;
 
+	/* If there is an empty-string label surrounding the label position */
+	/* then replace the position with the larger one and remove the	    */
+	/* empty label.							    */
+
+	sl = NULL;
+	for (lab = cifReadCellDef->cd_labels; lab != NULL; lab = lab->lab_next)
+	{
+	    if (lab->lab_text[0] == '\0')
+	    {
+		if ((GEO_SURROUND(&lab->lab_rect, &r)) && (lab->lab_type == type))
+		{
+		    r = lab->lab_rect;
+		    if (sl == NULL)
+			cifReadCellDef->cd_labels = lab->lab_next;
+		    else
+			sl->lab_next = lab->lab_next;
+		    if (cifReadCellDef->cd_lastLabel == lab)
+			cifReadCellDef->cd_lastLabel = sl;
+		    freeMagic((char *)lab);
+		    break;
+		}
+	    }
+	    sl = lab;
+	}
+
 	if (font < 0)
 	    lab = DBPutLabel(cifReadCellDef, &r, pos, textbody, type, flags);
 	else
@@ -905,18 +978,23 @@ calmaElementText()
 	if ((lab != NULL) && (cifnum >= 0) &&
 		(cifCurReadStyle->crs_labelSticky[cifnum] == LABEL_TYPE_PORT))
 	{
-	    Label *sl;
 	    int idx;
 
 	    /* No port information can be encoded in the GDS file, so	*/
 	    /* assume defaults, and assume that the port order is the	*/
-	    /* order in which labels arrive in the GDS stream.		*/
+	    /* order in which labels arrive in the GDS stream.  If	*/
+	    /* ports have the same text, then give them the same index.	*/
 
 	    i = -1;
 	    for (sl = cifReadCellDef->cd_labels; sl != NULL; sl = sl->lab_next)
 	    {
 		idx = sl->lab_flags & PORT_NUM_MASK;
 		if (idx > i) i = idx;
+		if ((idx > 0) && (sl != lab) && !strcmp(sl->lab_text, textbody))
+		{
+		    i = idx - 1;
+		    break;
+		}
 	    }
 	    i++;
 	    lab->lab_flags |= (PORT_NUM_MASK & i);
@@ -928,7 +1006,7 @@ calmaElementText()
     /* done with textbody */
     if (textbody != NULL) freeMagic(textbody);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -967,7 +1045,7 @@ calmaReadPath(pathheadpp, iscale)
     READRH(nbytes, rtype);
     if (nbytes < 0)
     {
-	calmaReadError("EOF when reading path.\n");
+	CalmaReadError("EOF when reading path.\n");
 	return (FALSE);
     }
     if (rtype != CALMA_XY)
@@ -994,7 +1072,7 @@ calmaReadPath(pathheadpp, iscale)
 	    }
 	}
 	if (ABS(path.cifp_x) > 0x0fffffff || ABS(path.cifp_y) > 0x0fffffff) {
-	    calmaReadError("Warning:  Very large point in path:  (%d, %d)\n",
+	    CalmaReadError("Warning:  Very large point in path:  (%d, %d)\n",
 		path.cifp_x, path.cifp_y);
 	}
 	if (feof(calmaInputFile))
@@ -1035,7 +1113,7 @@ calmaReadPath(pathheadpp, iscale)
     }
     return (*pathheadpp != NULL);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1066,12 +1144,21 @@ calmaLayerError(mesg, layer, dt)
     CalmaLayerType clt;
     HashEntry *he;
 
+    /* Ignore errors for cells that are marked as read-only, since  */
+    /* these are normally expected to have unhandled layer types,   */
+    /* since the purpose of read-only cells is to preserve exactly  */
+    /* layout in the cell which may not be represented in the tech  */
+    /* file.							    */
+
+    if ((cifReadCellDef->cd_flags & CDVENDORGDS) == CDVENDORGDS)
+	return;
+
     clt.clt_layer = layer;
     clt.clt_type = dt;
     he = HashFind(&calmaLayerHash, (char *) &clt);
     if (HashGetValue(he) == NULL)
     {
 	HashSetValue(he, (ClientData) 1);
-	calmaReadError("%s, layer=%d type=%d\n", mesg, layer, dt);
+	CalmaReadError("%s, layer=%d type=%d\n", mesg, layer, dt);
     }
 }
