@@ -57,6 +57,7 @@ bool CalmaDoLibrary = FALSE;	 /* If TRUE, do not output the top level */
 bool CalmaDoLabels = TRUE;	 /* If FALSE, don't output labels with GDS-II */
 bool CalmaDoLower = TRUE;	 /* If TRUE, allow lowercase labels. */
 bool CalmaFlattenArrays = FALSE; /* If TRUE, output arrays as individual uses */
+bool CalmaAddendum = FALSE;	 /* If TRUE, do not output readonly cell defs */
 
     /* Experimental stuff---not thoroughly tested (as of Sept. 2007)! */
 bool CalmaContactArrays = FALSE; /* If TRUE, output contacts as subcell arrays */
@@ -774,13 +775,6 @@ calmaProcessDef(def, outf, do_library)
 	    return (0);
     }
 
-    /*
-     * Output the definitions for any of our descendants that have
-     * not already been output.  Numbers are assigned to the subcells
-     * as they are output.
-     */
-    (void) DBCellEnum(def, calmaProcessUse, (ClientData) outf);
-
     /*
      * Check if this is a read-only file that is supposed to be copied
      * verbatim from input to output.  If so, do the direct copy.  If
@@ -799,6 +793,22 @@ calmaProcessDef(def, outf, do_library)
     DBPropGet(def, "GDS_END", &hasGDSEnd);
     filename = (char *)DBPropGet(def, "GDS_FILE", &isReadOnly);
 
+    /* When used with "calma addendum true", don't output the read-only	*/
+    /* cells.  This makes the library incomplete and dependent on the	*/
+    /* vendor libraries, so use with caution.				*/
+
+    if (isReadOnly && hasContent && CalmaAddendum) return (0);
+
+    /* Give some feedback to the user */
+    TxPrintf("   Writing cell %s\n", def->cd_name);
+
+    /*
+     * Output the definitions for any of our descendants that have
+     * not already been output.  Numbers are assigned to the subcells
+     * as they are output.
+     */
+    (void) DBCellEnum(def, calmaProcessUse, (ClientData) outf);
+
     if (isReadOnly && hasContent)
     {
 	char *buffer, *offptr, *retfilename;
@@ -1001,7 +1011,8 @@ calmaOutFunc(def, f, cliprect)
     /* Output all the tiles associated with this cell; skip temporary layers */
     GEO_EXPAND(&def->cd_bbox, CIFCurStyle->cs_radius, &bigArea);
     CIFErrorDef = def;
-    CIFGen(def, def, &bigArea, CIFPlanes, &DBAllTypeBits, TRUE, TRUE, (ClientData) f);
+    CIFGen(def, def, &bigArea, CIFPlanes, &DBAllTypeBits, TRUE, TRUE, FALSE,
+		(ClientData)f);
     if (!CIFHierWriteDisable)
 	CIFGenSubcells(def, &bigArea, CIFPlanes);
     if (!CIFArrayWriteDisable)
