@@ -1058,6 +1058,8 @@ CIFTechLine(sectionName, argc, argv)
 	newOp->co_opcode = CIFOP_CLOSE;
     else if (strcmp(argv[0], "bridge") == 0)
 	newOp->co_opcode = CIFOP_BRIDGE;
+    else if (strcmp(argv[0], "bridge-lim") == 0)
+	newOp->co_opcode = CIFOP_BRIDGELIM;
     else
     {
 	TechError("Unknown statement \"%s\".\n", argv[0]);
@@ -1107,6 +1109,26 @@ CIFTechLine(sectionName, argc, argv)
 	    newOp->co_client = (ClientData)bridge;
 	    break;
 
+	case CIFOP_BRIDGELIM:
+	    if (argc != 4) goto wrongNumArgs;
+	    newOp->co_distance = atoi(argv[1]);
+	    if (newOp->co_distance <= 0)
+	    {
+		TechError("Bridge distance must be greater than zero.\n");
+		goto errorReturn;
+	    }
+	    bridge = (BridgeData *)mallocMagic(sizeof(BridgeData));
+	    bridge->br_width = atoi(argv[2]);
+	    if (bridge->br_width <= 0)
+	    {
+		TechError("Bridge width must be greater than zero.\n");
+		freeMagic(bridge);
+		goto errorReturn;
+	    }
+	    cifParseLayers(argv[3], CIFCurStyle, &newOp->co_paintMask, &newOp->co_cifMask,FALSE);
+	    newOp->co_client = (ClientData)bridge;
+	    break;
+
 	case CIFOP_BLOATALL:
 	    if (argc != 3) goto wrongNumArgs;
 	    cifParseLayers(argv[1], CIFCurStyle, &newOp->co_paintMask,
@@ -1515,6 +1537,12 @@ cifComputeRadii(layer, des)
 
     for (op = layer->cl_ops; op != NULL; op = op->co_next)
     {
+	/* BBOX and NET operators should never be used hierarchically	*/
+	/* so ignore any grow/shrink operators that come after them.	*/
+
+	if (op->co_opcode == CIFOP_BBOX || op->co_opcode == CIFOP_NET)
+	    break;
+
 	/* If CIF layers are used, switch to the max of current
 	 * distances and those of the layers used.
 	 */
@@ -1572,8 +1600,10 @@ cifComputeRadii(layer, des)
 		break;
 
 	    case CIFOP_BRIDGE: break;
+	    case CIFOP_BRIDGELIM: break;
 	    case CIFOP_SQUARES: break;
 	    case CIFOP_SQUARES_G: break;
+
 	}
     }
 
@@ -1831,6 +1861,7 @@ CIFTechFinal()
 			case CIFOP_MAXRECT:
 			case CIFOP_NET:
 			    break;
+			case CIFOP_BRIDGELIM:
 			case CIFOP_BRIDGE:
 			    bridge = (BridgeData *)op->co_client;
 			    c = FindGCF(style->cs_scaleFactor,
@@ -1928,6 +1959,7 @@ CIFTechFinal()
 		case CIFOP_ANDNOT:
 		case CIFOP_SHRINK:
 		case CIFOP_CLOSE:
+		case CIFOP_BRIDGELIM:
 		case CIFOP_BRIDGE:
 		    needThisLayer = TRUE;
 		    break;
@@ -2143,7 +2175,11 @@ CIFGetContactSize(type, edge, spacing, border)
 
 		    /* Anything other than an OR function will break	*/
 		    /* the relationship between magic layers and cuts.	*/
-		    else if (sop->co_opcode != CIFOP_OR)
+		    /* NOTE:  Making an exception for AND_NOT, which is	*/
+		    /* used to distinguish between small and large via	*/
+		    /* areas.						*/
+		    else if ((sop->co_opcode != CIFOP_OR) &&
+				(sop->co_opcode != CIFOP_ANDNOT))
 			break;
 		}
     }
@@ -2355,6 +2391,7 @@ CIFTechOutputScale(n, d)
 			case CIFOP_MAXRECT:
 			case CIFOP_NET:
 			    break;
+			case CIFOP_BRIDGELIM:
 			case CIFOP_BRIDGE:
 			    bridge = (BridgeData *)op->co_client;
 			    bridge->br_width *= d;
@@ -2459,6 +2496,7 @@ CIFTechOutputScale(n, d)
 			    if (bloats->bl_distance[j] != 0)
 				bloats->bl_distance[j] /= lexpand;
 			break;
+		    case CIFOP_BRIDGELIM:
 		    case CIFOP_BRIDGE:
 			bridge = (BridgeData *)op->co_client;
 			bridge->br_width /= lexpand;
