@@ -4,21 +4,21 @@
  * Circuit extraction.
  * Command interface.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/extract/ExtMain.c,v 1.2 2001/01/12 22:12:37 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtMain.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -88,7 +88,7 @@ void extExtractStack();
 bool extContainsGeometry();
 bool extContainsCellFunc();
 bool extTimestampMisMatch();
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -153,7 +153,7 @@ ExtInit()
     /* Initialize the hash tables used in ExtLength.c */
     extLengthInit();
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -201,6 +201,7 @@ ExtAll(rootUse)
  * cell defs, in preparation for extracting a subtree
  * rooted at a particular def.
  */
+int
 extDefInitFunc(def)
     CellDef *def;
 {
@@ -213,6 +214,7 @@ extDefInitFunc(def)
  * if it hasn't already been pushed, and then recurse
  * on all that def's children.
  */
+int
 extDefPushFunc(use)
     CellUse *use;
 {
@@ -226,7 +228,7 @@ extDefPushFunc(use)
     (void) DBCellEnum(def, extDefPushFunc, (ClientData) 0);
     return (0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -234,10 +236,13 @@ extDefPushFunc(use)
  *
  * For each cell in the subtree rooted at rootUse->cu_def, make
  * sure that there are not two different nodes with the same label.
- * If there are, and either the label ends in a '#' or allNames is
- * TRUE, we generate unique names by appending a numeric suffix to
- * all but one of the offending labels.  Otherwise, if the label
- * doesn't end in a '!', we leave feedback.
+ * If there are, we generate unique names by appending a numeric
+ * suffix to all but one of the offending labels.
+ * If "option" is 1 (tagged mode), then only labels ending in the
+ * character "#" are forced to be unique.  If "option" is 2 (noports
+ * mode), then port labels are not forced to be unique.  Finally,
+ * if the label has been changed and doesn't end in a '!', we leave
+ * feedback.
  *
  * Results:
  *	None.
@@ -251,9 +256,9 @@ extDefPushFunc(use)
  */
 
 void
-ExtUnique(rootUse, allNames)
+ExtUnique(rootUse, option)
     CellUse *rootUse;
-    bool allNames;
+    int option;
 {
     CellDef *def;
     int nwarn;
@@ -277,13 +282,13 @@ ExtUnique(rootUse, allNames)
     {
 	def->cd_client = (ClientData) 0;
 	if (!SigInterruptPending)
-	    nwarn += extUniqueCell(def, allNames);
+	    nwarn += extUniqueCell(def, option);
     }
     StackFree(extDefStack);
     if (nwarn)
 	TxError("%d uncorrected errors (see the feedback info)\n", nwarn);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -361,7 +366,7 @@ extDefParentFunc(def)
 	if (parent->cu_parent)
 	    extDefParentFunc(parent->cu_parent);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -470,7 +475,7 @@ extContainsGeometry(def, allButUse, area)
     bool extContainsCellFunc();
     int pNum;
 
-    if (TiSrArea((Tile *) NULL, def->cd_planes[PL_CELL], area,
+    if (DBSrCellPlaneArea(def->cd_cellPlane, area,
 			extContainsCellFunc, (ClientData) allButUse))
 	return (TRUE);
 
@@ -484,17 +489,11 @@ extContainsGeometry(def, allButUse, area)
 }
 
 bool
-extContainsCellFunc(tile, allButUse)
-    Tile *tile;
+extContainsCellFunc(use, allButUse)
+    CellUse *use;
     CellUse *allButUse;
 {
-    CellTileBody *ctb;
-
-    for (ctb = (CellTileBody *) TiGetBody(tile); ctb; ctb = ctb->ctb_next)
-	if (ctb->ctb_use != allButUse)
-	    return (TRUE);
-
-    return (FALSE);
+    return (use != allButUse) ? TRUE : FALSE;
 }
 
 int
@@ -502,7 +501,7 @@ extContainsPaintFunc()
 {
     return (1);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -587,8 +586,11 @@ extTimestampMisMatch(def)
     FILE *extFile;
     bool ret = TRUE;
     int stamp;
+    bool doLocal;
 
-    extFile = extFileOpen(def, (char *) NULL, "r", (char **) NULL);
+    doLocal = (ExtOptions & EXT_DOLOCAL) ? TRUE : FALSE;
+
+    extFile = extFileOpen(def, (char *) NULL, "r", doLocal, (char **) NULL);
     if (extFile == NULL)
 	return (TRUE);
 
@@ -601,7 +603,7 @@ closeit:
     (void) fclose(extFile);
     return (ret);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -627,7 +629,6 @@ closeit:
 void
 extExtractStack(stack, doExtract, rootDef)
     Stack *stack;
-    bool doExtract;
     CellDef *rootDef;
 {
     int fatal = 0, warnings = 0;
