@@ -3,16 +3,16 @@
  *
  * Read in a technology file.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -34,7 +34,6 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "windows/windows.h"
 #include "utils/malloc.h"
 
-global char *TechDefault = NULL;
 global int  TechFormatVersion;
 global bool TechOverridesDefault;
 
@@ -106,7 +105,7 @@ techSection *techSectionFree;		/* Pointer to next free section */
 techSection *techCurrentSection;	/* Pointer to current section */
 
 techSection *techFindSection();
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -373,7 +372,7 @@ TechAddClient(sectionName, init, proc, final, prevSections, pSectionID, opt)
 	tcl->tc_next = tcp;
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  * TechLoad --
@@ -527,8 +526,8 @@ TechLoad(filename, initmask)
 #endif
 	ExtTechInit();
 	DRCTechInit();
-	MZTechInit();	
-	
+	MZTechInit();
+
             /* Changing number of planes requires handling on every     */
             /* celldef.  So we need to save the original number of      */
             /* planes to see if it shrinks or expands.                  */
@@ -700,7 +699,13 @@ skipsection:
     }
     if (fstack) fclose(fstack->file);
 
-    if (retval == TRUE)
+    /* Note:  If filename is NULL, then individual sections are	*/
+    /* being reloaded, and it is the responsibility of the	*/
+    /* calling routine to invoke any exit function specific to	*/
+    /* that section (e.g., DRCTechScale() when loading a new	*/
+    /* DRC style).						*/
+
+    if ((filename != NULL) && (retval == TRUE))
     {
 	/* If internal scalefactor is not the default 1:1, then we  */
 	/* need to scale the techfile numbers accordingly.          */
@@ -748,7 +753,7 @@ skipsection:
 	if (saveNumPlanes != DBNumPlanes)
 	    DBCellSrDefs(0, changePlanesFunc, (ClientData) &saveNumPlanes);
     }
-    else
+    else if (retval == FALSE)
     {
 	/* On error, remove any existing technology file name */
 	freeMagic(TechFileName);
@@ -757,7 +762,7 @@ skipsection:
 
     return (retval);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -800,7 +805,7 @@ TechError(char *fmt, ...)
     va_end(args);
 }
 
-
+
 /* ================== Functions local to this module ================== */
 
 /*
@@ -838,7 +843,7 @@ techFindSection(sectionName)
     }
     return ((techSection *) NULL);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -947,7 +952,7 @@ start:
 	 * Grab up characters to the end of the token.  Any character
 	 * preceded by a backslash is taken literally.
 	 */
-	
+
 	while (*get != '\0')
 	{
 	    if (inquote)
@@ -972,7 +977,7 @@ start:
 	 */
 	if (get == argv[argc])
 	    break;
-	
+
 	/* Terminate the token and advance over the terminating character. */
 
 	if (*get != '\0') get++;	/* Careful!  could be at end of line! */
